﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;


namespace MVRPlugin {
	
	public class LightToggler : MVRScript 
	
	{
        #region PluginInfo    
        public string pluginAuthor = "Huha"; // fixed by Spacedog )
        public string pluginName = "Light_toggler";
        public string pluginVersion = "1";
        public string pluginDate = "02/07" + "/2023";
        public string pluginDescription = @"to turn on and off emissions";
        #endregion	

        protected UIDynamicSlider lightIntensitySlider;
        protected UIDynamicToggle diffOffsetSlider;

        protected List<Light> origLights = new List<Light>();
        protected List<Material> emittingMaterials = new List<Material>();
        protected List<Light> customLights = new List<Light>();
        protected JSONStorableFloat jLightIntensity;		
        private JSONStorableBool jLightEnabled;
		protected List<Renderer> renderers = new List<Renderer>();
				

		public override void Init() {
			try {
				BuildUIControls();
				StartCoroutine(WaitForAssetBundle());
              // A tiny bit of help to give some info to your fellow creator and/or player
                JSONStorableString helpText = new JSONStorableString("About",
                    "<color=#000><size=35><b>About</b></size></color>\n\n" +
                    "<color=#333>" +
                    "Light Toggler Pugin.\n\n" +					
                    "Credits to Huha for original script.\n\n" +
                    "fixed loading issue by Spacedog.\n\n" +					
                    "this plugin switches on or off unity light emissions.\n\n" +
                    "in any unity Asset that uses light emmissions.\n\n" +					
                    "</color>" );
               UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 1100.0f;					

			} catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		protected void BuildUIControls() {
            jLightEnabled = new JSONStorableBool("lightEnabled", true, SetLightIntensity);
            jLightEnabled.storeType = JSONStorableParam.StoreType.Full;
            RegisterBool(jLightEnabled);
            CreateToggle(jLightEnabled, false);		
        }

		protected bool IsValidMaterial(Material mat) {
			return true;
		}

		protected IEnumerator WaitForAssetBundle() {
			List<Light> allLights = new List<Light>();
            List<Renderer> allRenderers = new List<Renderer>();

            float waitForSeconds = 0.5f;
            while (allRenderers.Count < 1 && allLights.Count < 1)
            {
                containingAtom.GetComponentsInChildren<Renderer>(false, allRenderers);
				containingAtom.GetComponentsInChildren<Light>(false, allLights);
                yield return new WaitForSeconds(0.5f);
            }
			GetLights(allLights);
            GetMaterials(allRenderers);
            SetLightIntensity(jLightEnabled);

        }

        protected void GetMaterials(List<Renderer> renderers)
        {
            foreach(Renderer renderer in renderers)
            {
                Material[] mats = renderer.materials;
                for (int i = 0; i < mats.Length; i++)
                {
                    if (mats[i].IsKeywordEnabled("_EMISSION"))
                    {
                        emittingMaterials.Add(mats[i]);
                    }
                }
            }

        }

        protected void GetLights(List<Light> lights) {
            if (lights.Count == 0)
            {
                return;
            }
            foreach (Light light in lights)
            {
                customLights.Add(light);
            }
        }


		void OnDestroy() {
		}

        void turnLightsOff()
        {
            foreach (Light light in customLights)
            {
                light.enabled = false;
            }
            foreach(Material mat in emittingMaterials)
            {
                mat.DisableKeyword("_EMISSION");
            }
        }

        void turnLightsOn()
        {
            foreach (Light light in customLights)
            {
                light.enabled = true;
            }

            foreach (Material mat in emittingMaterials)
            {
                mat.EnableKeyword("_EMISSION");
            }
        }

		protected void SetLightIntensity(JSONStorableBool jl) {
            {
                if (jl.val == true)
                {
                    turnLightsOn();
                } else
                {
                    turnLightsOff();
                }
            }
		}

	}
}